import logging
from typing import Callable
from urllib.parse import urlparse
from pathlib import Path
import requests

from instagrapi import Client
from instagrapi.exceptions import VideoNotDownload

class WBClient(Client):
	"""
	patched instagrapi
	"""
	def __init__(self) -> None:
		super().__init__()
		self.progress_callback = None

	def set_progress_callback(self, callback: Callable[[int | None, int, Path], None]) -> None:
		if not callback or not callable(callback):
			raise TypeError("Progress callback must be callable")
		self.progress_callback = callback

	def video_download_by_url(self, url: str, filename: str = "", folder: Path = "") -> Path:
		fname = urlparse(url).path.rsplit("/", 1)[1]
		filename = f"{filename}.{fname.rsplit('.', 1)[1]}" if filename else fname
		path = Path(folder or Path.cwd()) / filename

		logging.info("Downloading video from %s to %s", url, path)

		response = requests.get(url, stream=True, timeout=self.request_timeout)
		response.raise_for_status()

		content_length = 0
		try:
			content_length = int(response.headers.get("Content-Length", 0))
		except (TypeError, ValueError):
			logging.warning("Content-Length header is missing or invalid.")
			content_length = 0

		downloaded = 0
		with open(path, "wb") as f:
			for chunk in response.iter_content(chunk_size=8192):
				if chunk:
					f.write(chunk)
					downloaded += len(chunk)
					if self.progress_callback:
						try:
							self.progress_callback(
								total=content_length or None,
								bytes_transferred=downloaded,
								path=path
							)
						except Exception as e:
							logging.warning("Progress callback raised an exception!", exc_info=e)


		if content_length and downloaded != content_length:
			raise VideoNotDownload(
				f'Broken file "{path}" (expected {content_length}, got {downloaded})'
			)

		return path.resolve()


import logging
from multiprocessing import Pipe
from pyrogram import Client
from warp_beacon.telegram.progress_bar import ProgressBar
from warp_beacon.telegram.types import ReportType

class DownloadStatus(object):
	status_pipe = None
	child_conn = None
	client = None
	progress_bars = None

	def __init__(self, client: Client) -> None:
		self.progress_bars = {}
		self.client = client
		self.status_pipe, self.child_conn = Pipe()

	async def handle_message(self, msg: dict, progress_bar: ProgressBar) -> None:
		progress_bar.progress_callback(
			current=msg.get("current", 0),
			total=msg.get("total", 0),
			message_id=msg.get("message_id", 0),
			chat_id=msg.get("chat_id", 0),
			operation="Downloading",
			report_type=msg.get("report_type", ReportType.PROGRESS)
		)

	def on_status(self) -> None:
		msg = self.status_pipe.recv()
		if not msg:
			logging.warning("Empty status message!")
			return
		logging.info("Received pipe message: %s", msg)
		message_id = msg.get("message_id", 0)
		chat_id = msg.get("chat_id", 0)
		a_key = f"{message_id}:{chat_id}"
		progress_bar = None
		if a_key not in self.progress_bars:
			progress_bar = ProgressBar(self.client)
			self.progress_bars[a_key] = progress_bar
		else:
			progress_bar = self.progress_bars[a_key]
		task = self.client.loop.create_task(self.handle_message(msg, progress_bar))
		if msg.get("completed", False):
			task.add_done_callback(lambda _: self.progress_bars.pop(a_key, None))